<?php
/**
 * @file
 * Rules specific datas the LimeSurvey Sync module.
 */

/**
 * Implements hook_rules_event_info().
 * @ingroup rules *
 */
function limesurvey_sync_rules_event_info() {
  $default = array(
    'group' => t('LimeSurvey Sync'),
    'module' => 'limesurvey_sync',
    'variables' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('LimeSurvey entity.'),
      ),
    ),
  );
  return array(
    'limesurvey_sync_rules_event_synchronize_dp2ls' => array(
      'label' => t('Synchronize Drupal to LimeSurvey'),
    ) + $default,
    'limesurvey_sync_rules_event_synchronize_ls2dp' => array(
      'label' => t('Synchronize LimeSurvey to Drupal'),
    ) + $default,
  );
}

/**
 * Implements hook_rules_condition_info().
 * @ingroup rules *
 */
function limesurvey_sync_rules_condition_info() {
  $conditions = array();
  $default = array(
    'group' => t('LimeSurvey Sync'),
    'module' => 'limesurvey_sync',
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('The LimeSurvey entity'),
        'description' => t('Specifies the entity for which to evaluate the condition.'),
      ),
    ),
  );
  $conditions['limesurvey_sync_rules_condition_is_synchronized'] = $default + array(
    'label' => t('LimeSurvey entity is synchronized'),
  );
  $conditions['limesurvey_sync_rules_condition_is_ls_entity'] = $default + array(
    'label' => t('Entity is a LimeSurvey entity'),
  );
  $conditions['limesurvey_sync_rules_condition_is_type_of'] = $default + array(
    'label' => t('LimeSurvey entity is type of'),
  );
  $conditions['limesurvey_sync_rules_condition_is_type_of']['parameter']['lss_entity_type'] = array(
    'type' => 'token',
    'label' => t('LimeSurvey entity type'),
    'description' => t('The type of the checked LimeSurvey entity.'),
    'options list' => 'limesurvey_sync_rules_lss_entity_type_options',
    'restriction' => 'input',
  );

  return $conditions;
}

/**
 * return TRUE if an answer is synchronized.
 */
function limesurvey_sync_rules_condition_is_synchronized($entity) {
  // $entity is provided trought the entity_metada_wrapper. we fix it.
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  if ($entity instanceof EntityDrupalWrapper) {
    $entity = $entity->value();
  }
  $synchronize_set = $entity->ls_synchronize();
  return (isset($synchronize_set['ok']));
}

/**
 * return TRUE if the entity is a LimeSurvey entity
 */
function limesurvey_sync_rules_condition_is_ls_entity($entity) {
  // $entity is provided trought the entity_metada_wrapper. we fix it.
  if ($entity instanceof EntityDrupalWrapper) {
    $entity = $entity->value();
  }
  return (isset($entity->lss_entity_type));
}

/**
 * return TRUE if the entity is a type of the provided type.
 *
 * Allowed types are 'answer', 'token', 'response', 'survey'.
 */
function limesurvey_sync_rules_condition_is_type_of($entity, $lss_entity_type) {
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  // $entity is provided trought the entity_metada_wrapper. we fix it.
  if ($entity instanceof EntityDrupalWrapper) {
    $entity = $entity->value();
  }
  $result = (isset($entity->lss_entity_type) && $entity->lss_entity_type == $lss_entity_type);
  return (isset($entity->lss_entity_type) && $entity->lss_entity_type == $lss_entity_type);
}

function limesurvey_sync_rules_lss_entity_type_options() {
  return array(
    'answer' => t('Answer'),
    'survey' => t('Survey'),
    'token' => t('LimeSurvey token invitation'),
  );
}

/**
 * Implements hook_rules_action_info().
 *
 */
function limesurvey_sync_rules_action_info() {
  $actions = array();
  $default = array(
    'group' => t('LimeSurvey Sync'),
    'module' => 'limesurvey_sync',
    'parameter' => array(
      'entity' => array(
        'type' => 'entity',
        'label' => t('The LimeSurvey entity'),
        'description' => t('Specifies the LimeSurvey entity for which the action will be fired.'),
      ),
    ),
  );

  $actions['limesurvey_sync_rules_action_synchronize'] = $default + array(
    'label' => t('Synchronize the LimeSurvey entity'),
  );
  $actions['limesurvey_sync_rules_action_synchronize']['parameter']['way'] = array(
    'type' => 'token',
    'label' => t('Synchronization direction'),
    'description' => t('If datas are different between Drupal and LimeSurvey, choose the one which will overwrite the other'),
    'options list' => 'limesurvey_sync_rules_synchronize_way_options',
    'default value' => 'ls2dp',
    'restriction' => 'input',
  );

  $actions['limesurvey_sync_rules_action_recent_open_token'] = $default + array(
    'label' => t('Recent opened token for a user'),
  );
  $actions['limesurvey_sync_rules_action_recent_open_token']['parameter'] = array(
    'lss_survey' => array(
      'type' => 'limesurvey_sync_survey',
      'label' => t('Survey'),
      //'options list' => 'limesurvey_sync_survey_sids_options'
    ),
    'account' => array(
      'type' => 'user',
      'label' => t('User'),
      'description' => t('Specifies the user for which the action will be fired.'),
    ),
  );
  $actions['limesurvey_sync_rules_action_recent_open_token']['provides'] = array(
    'token' => array(
      'type' => 'entity',
      'label' => t('LimeSurvey Token entity'),
      'description' => t('The most recent opened LimeSurvey Token for this user.'),
      //'options list' => 'limesurvey_sync_survey_sids_options'
    ),
  );

  //foreach (entity_get_info() as $entity_type => $info) {
//  if (!empty($info['customizable entities']['lss_entity_type']) && $info['customizable entities']['lss_entity_type'] == 'token') {
//    $rule_name = limesurvey_sync_api_rules_names($entity_type, 'token', 'recent_token');
//    $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : '';
  //$actions[$rule_name] = $default + array('label' => t('Recent opened token for a user'));
//  }
  //}

  return $actions;
}

function limesurvey_sync_rules_action_synchronize($entity, $way) {
  if (!limesurvey_sync_rules_condition_is_ls_entity($entity)) {
    // This is not a LimeSurvey entity.
    return FALSE;
  }
  // $entity is provided trought entity_metada_wrapper().
  if ($way == 'ls2dp') {
    $entity->need_save_ls2dp = TRUE;
  }
  elseif ($way == 'dp2ls') {
    $entity->need_save_dp2ls->set(TRUE);
  }
  $entity->save();
}

function limesurvey_sync_rules_synchronize_way_options() {
  return array(
    'ls2dp' => t('Synchronize from LimeSurvey to Drupal (recommanded)'),
    'dp2ls' => t('Synchronize from Drupal to LimeSurvey'),
  );
}

function limesurvey_sync_rules_action_recent_open_token($survey, $account) {
  $token_entityname = limesurvey_sync_api_entityname('token', $survey->ls_sid);
  return limesurvey_sync_api_token_retreive_recent_open_token($token_entityname, $account->uid);
}
