<?php
/**
 * @file
 * Default rules for the LimeSurvey Sync module.
 */

/**
 * Implements hook_default_rules_configuration().
 */
function limesurvey_sync_default_rules_configuration() {
  $config = array();
  foreach (entity_get_info() as $entity_type => $info) {
    if (!empty($info['customizable entities']['lss_entity_type']) && $info['customizable entities']['lss_entity_type'] == 'token') {
      $rule_name = limesurvey_sync_api_rules_names($entity_type, 'token', 'dp2ls');
      $rule_export = limesurvey_sync_rules_export_token_dp2ls($entity_type);
      if ($rule_name && $rule_export) {
        $config[$rule_name] = rules_import($rule_export);
      }
      $rule_name = limesurvey_sync_api_rules_names($entity_type, 'token', 'ls2dp');
      $rule_export = limesurvey_sync_rules_export_token_ls2dp($entity_type);
      if ($rule_name && $rule_export) {
        $config[$rule_name] = rules_import($rule_export);
      }
    }
  }
  return $config;
}

function limesurvey_sync_rules_export_token_dp2ls($token_entityname) {
  if ($token_entityname && $info = entity_get_info($token_entityname)) {
    $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : '';
    $rule_name = limesurvey_sync_api_rules_names($token_entityname, 'token', 'dp2ls');
    return '{ "' . $rule_name . '" : {
    "LABEL" : "LimeSurvey Sync token import user infos from DP to LS (' . $ls_sid . ')",
    "PLUGIN" : "reaction rule",
    "OWNER" : "rules",
    "TAGS" : [ "limesurvey sync" ],
    "REQUIRES" : [ "limesurvey_sync", "rules" ],
    "ON" : { "limesurvey_sync_rules_event_synchronize_dp2ls" : [] },
    "IF" : [
      { "entity_is_of_type" : { "entity" : [ "entity" ], "type" : "' . $token_entityname . '" } },
      { "limesurvey_sync_rules_condition_is_type_of" : { "entity" : [ "entity" ], "lss_entity_type" : "token" } },
      { "NOT data_is_empty" : { "data" : [ "entity:lss-uid" ] } },
      { "NOT data_is_empty" : { "data" : [ "entity:lss-uid:mail" ] } }
    ],
    "DO" : [
      { "data_set" : { "data" : [ "entity:ls-email" ], "value" : [ "entity:lss-uid:mail" ] } }
    ]
  }
}';
  }
  else {
    return FALSE;
  }
}

function limesurvey_sync_rules_export_token_ls2dp($token_entityname) {
  if ($token_entityname && entity_get_info($token_entityname)) {
    $rule_name = limesurvey_sync_api_rules_names($token_entityname, 'token', 'ls2dp');
    $ls_sid = (!empty($info['customizable entities']['ls_sid'])) ? $info['customizable entities']['ls_sid'] : '';
    return '{ "' . $rule_name . '" : {
    "LABEL" : "LimeSurvey Sync token import token infos from LS to DP (' . $ls_sid . ')",
    "PLUGIN" : "reaction rule",
    "ACTIVE" : false,
    "OWNER" : "rules",
    "TAGS" : [ "limesurvey sync" ],
    "REQUIRES" : [ "rules", "limesurvey_sync" ],
    "ON" : { "limesurvey_sync_rules_event_synchronize_ls2dp" : [] },
    "IF" : [
      { "entity_is_of_type" : { "entity" : [ "entity" ], "type" : "' . $token_entityname . '" } },
      { "limesurvey_sync_rules_condition_is_type_of" : { "entity" : [ "entity" ], "lss_entity_type" : "token" } },
      { "NOT data_is_empty" : { "data" : [ "entity:ls-email" ] } },
      { "data_is_empty" : { "data" : [ "entity:lss-uid" ] } }
    ],
    "DO" : [
      { "entity_create" : {
        "USING" : {
          "type" : "user",
            "param_name" : [ "entity:ls-lastname" ],
            "param_mail" : [ "entity:ls-email" ]
          },
          "PROVIDE" : { "entity_created" : { "new_account" : "New User" } }
        }
      },
      { "data_set" : { "data" : [ "entity:lss-uid" ], "value" : [ "new-account" ] } }
    ]
  }
}';
  }
  else {
    return FALSE;
  }
}
